{ **************************************************************************** }
{ *                                                                          * }
{ * DONKEY KONG CIRCUS (Nintendo, Game & Watch Panorama Screen) Simulator    * }
{ * by Luca "MADrigal" Antignano, February 2000 (revision: August 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:          TImage;
    btn_game_menu_background:   TImage;

    im_backdrop:                TImage;
    im_backdrop_logo:           TImage;

    // electronic game
    im_background_bottom:       TImage;     im_background_front:      TImage;
                                            im_background_back:       TImage;

                                            im_background_screen_top: TImage;

    btn_open:                   TImage;

    btn_left_down:              TImage;     btn_left_top:             TImage;
    btn_right_down:             TImage;     btn_right_top:            TImage;

    btn_game_a_down:            TImage;     btn_game_a_top:           TImage;
    btn_game_b_down:            TImage;     btn_game_b_top:           TImage;
    btn_time_down:              TImage;     btn_time_top:             TImage;

    btn_acl_down:               TImage;     btn_acl_top:              TImage;
                                            btn_alarm_top:            TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (background, alarm, time/score, game a/b, etc.)
    im_game_background: TImage;

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage;

    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_digit_1: TImage; im_panel_digit_2: TImage; im_panel_digit_3: TImage; im_panel_digit_4: TImage;
    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_hero_1_body: TImage; im_hero_1_hand_down: TImage; im_hero_1_hand_up: TImage; im_hero_1_leg_left_up: TImage; im_hero_1_leg_left_down: TImage; im_hero_1_leg_right_up: TImage; im_hero_1_leg_right_down: TImage; im_hero_1_base: TImage;
    im_hero_2_body: TImage; im_hero_2_hand_down: TImage; im_hero_2_hand_up: TImage; im_hero_2_leg_left_up: TImage; im_hero_2_leg_left_down: TImage; im_hero_2_leg_right_up: TImage; im_hero_2_leg_right_down: TImage; im_hero_2_base: TImage;
    im_hero_3_body: TImage; im_hero_3_hand_down: TImage; im_hero_3_hand_up: TImage; im_hero_3_leg_left_up: TImage; im_hero_3_leg_left_down: TImage; im_hero_3_leg_right_up: TImage; im_hero_3_leg_right_down: TImage; im_hero_3_base: TImage;
    im_hero_4_body: TImage; im_hero_4_hand_down: TImage; im_hero_4_hand_up: TImage; im_hero_4_leg_left_up: TImage; im_hero_4_leg_left_down: TImage; im_hero_4_leg_right_up: TImage; im_hero_4_leg_right_down: TImage; im_hero_4_base: TImage;

    im_item_1_1: TImage; im_item_1_2: TImage; im_item_1_3: TImage; im_item_1_4: TImage; im_item_1_5: TImage; im_item_1_6: TImage; im_item_1_7: TImage;
    im_item_2_1: TImage; im_item_2_2: TImage; im_item_2_3: TImage; im_item_2_4: TImage; im_item_2_5: TImage; im_item_2_6: TImage; im_item_2_7: TImage;
    im_item_3_1: TImage; im_item_3_2: TImage; im_item_3_3: TImage; im_item_3_4: TImage; im_item_3_5: TImage; im_item_3_6: TImage; im_item_3_7: TImage;

    im_flame_1_1: TImage; im_flame_1_2: TImage; im_flame_1_3: TImage; im_flame_1_4: TImage; im_flame_1_5: TImage; im_flame_1_6: TImage; im_flame_1_7: TImage;
    im_flame_2_1: TImage; im_flame_2_2: TImage; im_flame_2_3: TImage; im_flame_2_4: TImage; im_flame_2_5: TImage; im_flame_2_6: TImage; im_flame_2_7: TImage;

    im_character_2_1: TImage; im_character_2_2: TImage;

{ * TIMERS                                                                   * }
    Timer_Chance_Time: TTimer;
    Timer_Clock:       TTimer;
    Timer_Game:        TTimer;
    Timer_Game_Over:   TTimer;
    Timer_Game_Start:  TTimer;
    Timer_Hero_Hand_1: TTimer;
    Timer_Hero_Hand_2: TTimer;
    Timer_Hero_Hand_3: TTimer;
    Timer_Hero_Hand_4: TTimer;
    Timer_Miss_1:      TTimer;
    Timer_Miss_2:      TTimer;
    Timer_Miss_3:      TTimer;
    Timer_Score_Blink: TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Hero             (b: boolean);
    procedure Vid_Show_All_Items            (b: boolean);
    procedure Vid_Show_All_Flames           (b: boolean);
    procedure Vid_Show_All_Character_2      (b: boolean);

    procedure Vid_Show_Panel_Miss           (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

    procedure Vid_Show_Hero                 (var p, f: integer; b: boolean);
    procedure Vid_Show_Items;

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_background_bottomMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_background_bottomMouseMove (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_openMouseDown             (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_open_down;

    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Chance_Time;
    procedure Gam_Check_Collision_Hand_Item;
    procedure Gam_Check_Collision_Item_Hand;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Chance_TimeTimer        (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_Hero_Hand_1Timer        (Sender: TObject);
    procedure Timer_Hero_Hand_2Timer        (Sender: TObject);
    procedure Timer_Hero_Hand_3Timer        (Sender: TObject);
    procedure Timer_Hero_Hand_4Timer        (Sender: TObject);
    procedure Timer_Miss_1Timer             (Sender: TObject);
    procedure Timer_Miss_2Timer             (Sender: TObject);
    procedure Timer_Miss_3Timer             (Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Donkey Kong Circus (Nintendo, Game & Watch Panorama Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\DKongCrP50';                                 // registry key

  HINT_CLICK  = 'click or press';
  HINT_OPEN   = 'to open the game';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';

{}MENU_ITEMS = 13;

  TIMER_GAME_NORMAL       = 650;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_beep1,
  pfs_beep2,
  pfs_beep3,
  pfs_beep4,
  pfs_chance,
  pfs_miss1,
  pfs_miss2,
  pfs_miss3,
  pfs_gameover,
  pfs_start,
  pfs_step:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_miss:          array[1..3]       of TImage;                         // miss marks panel
  aIm_panel_alarm:         array[1..3]       of TImage;                         // alarm panel
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time

  aIm_hero:                array[1..4]       of record                          // hero sprites
                                                  body,
                                                  hand_up,
                                                  hand_down,
                                                  leg_left_up,
                                                  leg_left_down,
                                                  leg_right_up,
                                                  leg_right_down,
                                                  base:           TImage;
                                                end;

  aIm_character_2:         array[1..2]       of TImage;                         // "Character No. 2" sprites
  aIm_item:                array[1..3, 1..7] of TImage;                         // normal items sprites
  aIm_flame:               array[1..2, 1..7] of TImage;                         // flaming items sprites

{ * arrays of integers * }
  aItem,                                                                        // items positions
  aItemTarget:             array[1..3]       of integer;                        // items target Y coordinates

{ * arrays of timers * }
  aT_hand:                 array[1..4]       of TTimer;                         // timers that make Donkey Kong's hand disappear

{ * arrays of mixed types *}
  aKeys:                   array[1..6]       of record                          // keys management (values)
                                                  int:  integer;
                                                  word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=closed, 1/2=ACL 1/2, 3=alarm, 4=time, 5/7=hiscore A/B, 6/8=game A/B, 9=game over
  iScore,                                                                       // score=0-999
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999
  iScoreMultiplier,                                                             // given points=1 (normal) or 2 (chance-time)
  iScoreNextChance,                                                             // 300 and every further 1000 points

  iPosHeroX,                                                                    // 1-4
  iMiss,                                                                        // 0-3

  iMaxRandomFlames                                                              // 9-12 (lower number = higher chances that a note is generated)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bControls,
  bKeyPressed
                         : boolean;

  // Game variables: loops
  iLoopChanceTime,
  iLoopGame,
  iLoopGameOver,
  iLoopGameStart,
  iLoopMiss1,
  iLoopMiss2,
  iLoopMiss3,
  iLoopMiss3FlameX,
  iLoopScoreBlink
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3;
  aIm_panel_digit_pos[1] := im_panel_digit_1; aIm_panel_digit_pos[2] := im_panel_digit_2; aIm_panel_digit_pos[3] := im_panel_digit_3; aIm_panel_digit_pos[4] := im_panel_digit_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_hero[1].body           := im_hero_1_body;           aIm_hero[2].body           := im_hero_2_body;           aIm_hero[3].body           := im_hero_3_body;           aIm_hero[4].body           := im_hero_4_body;
  aIm_hero[1].hand_up        := im_hero_1_hand_up;        aIm_hero[2].hand_up        := im_hero_2_hand_up;        aIm_hero[3].hand_up        := im_hero_3_hand_up;        aIm_hero[4].hand_up        := im_hero_4_hand_up;
  aIm_hero[1].hand_down      := im_hero_1_hand_down;      aIm_hero[2].hand_down      := im_hero_2_hand_down;      aIm_hero[3].hand_down      := im_hero_3_hand_down;      aIm_hero[4].hand_down      := im_hero_4_hand_down;
  aIm_hero[1].leg_left_up    := im_hero_1_leg_left_up;    aIm_hero[2].leg_left_up    := im_hero_2_leg_left_up;    aIm_hero[3].leg_left_up    := im_hero_3_leg_left_up;    aIm_hero[4].leg_left_up    := im_hero_4_leg_left_up;
  aIm_hero[1].leg_left_down  := im_hero_1_leg_left_down;  aIm_hero[2].leg_left_down  := im_hero_2_leg_left_down;  aIm_hero[3].leg_left_down  := im_hero_3_leg_left_down;  aIm_hero[4].leg_left_down  := im_hero_4_leg_left_down;
  aIm_hero[1].leg_right_up   := im_hero_1_leg_right_up;   aIm_hero[2].leg_right_up   := im_hero_2_leg_right_up;   aIm_hero[3].leg_right_up   := im_hero_3_leg_right_up;   aIm_hero[4].leg_right_up   := im_hero_4_leg_right_up;
  aIm_hero[1].leg_right_down := im_hero_1_leg_right_down; aIm_hero[2].leg_right_down := im_hero_2_leg_right_down; aIm_hero[3].leg_right_down := im_hero_3_leg_right_down; aIm_hero[4].leg_right_down := im_hero_4_leg_right_down;
  aIm_hero[1].base           := im_hero_1_base;           aIm_hero[2].base           := im_hero_2_base;           aIm_hero[3].base           := im_hero_3_base;           aIm_hero[4].base           := im_hero_4_base;

  aIm_item[1, 1] := im_item_1_1; aIm_item[2, 1] := im_item_2_1; aIm_item[3, 1] := im_item_3_1;
  aIm_item[1, 2] := im_item_1_2; aIm_item[2, 2] := im_item_2_2; aIm_item[3, 2] := im_item_3_2;
  aIm_item[1, 3] := im_item_1_3; aIm_item[2, 3] := im_item_2_3; aIm_item[3, 3] := im_item_3_3;
  aIm_item[1, 4] := im_item_1_4; aIm_item[2, 4] := im_item_2_4; aIm_item[3, 4] := im_item_3_4;
  aIm_item[1, 5] := im_item_1_5; aIm_item[2, 5] := im_item_2_5; aIm_item[3, 5] := im_item_3_5;
  aIm_item[1, 6] := im_item_1_6; aIm_item[2, 6] := im_item_2_6; aIm_item[3, 6] := im_item_3_6;
  aIm_item[1, 7] := im_item_1_7; aIm_item[2, 7] := im_item_2_7; aIm_item[3, 7] := im_item_3_7;

  aIm_flame[1, 1] := im_flame_1_1; aIm_flame[2, 1] := im_flame_2_1;
  aIm_flame[1, 2] := im_flame_1_2; aIm_flame[2, 2] := im_flame_2_2;
  aIm_flame[1, 3] := im_flame_1_3; aIm_flame[2, 3] := im_flame_2_3;
  aIm_flame[1, 4] := im_flame_1_4; aIm_flame[2, 4] := im_flame_2_4;
  aIm_flame[1, 5] := im_flame_1_5; aIm_flame[2, 5] := im_flame_2_5;
  aIm_flame[1, 6] := im_flame_1_6; aIm_flame[2, 6] := im_flame_2_6;
  aIm_flame[1, 7] := im_flame_1_7; aIm_flame[2, 7] := im_flame_2_7;

  aIm_character_2[1] := im_character_2_1;
  aIm_character_2[2] := im_character_2_2;

  // arrays of timers
  aT_hand[1] := Timer_Hero_Hand_1; aT_hand[2] := Timer_Hero_Hand_2; aT_hand[3] := Timer_Hero_Hand_3; aT_hand[4] := Timer_Hero_Hand_4;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"   (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"  (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 6 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_open.Hint                := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_OPEN;   // "Open"

  btn_game_a_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint             := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint           := HINT_ALARM;                                                     // "Alarm"

  btn_left_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Left"
  btn_right_top.Hint           := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bKeyPressed := false;                                                         // prevents hero movement

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_beep1);    FSOUND_Sample_Free(pfs_beep2);
  FSOUND_Sample_Free(pfs_beep3);    FSOUND_Sample_Free(pfs_beep4);
  FSOUND_Sample_Free(pfs_chance);   FSOUND_Sample_Free(pfs_miss1);
  FSOUND_Sample_Free(pfs_miss2);    FSOUND_Sample_Free(pfs_miss3);
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_start);
  FSOUND_Sample_Free(pfs_step);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  for i := 1 to 3 do
    aIm_panel_alarm[i].Visible := b;

  im_game_background.Visible := b;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  Vid_Show_All_Hero(b);
  Vid_Show_All_Items(b);
  Vid_Show_All_Flames(b);
  Vid_Show_All_Character_2(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all hero's sprites                                            }
  var
    i: integer;
begin
  for i := 1 to 4 do
    begin
      aIm_hero[i].body.Visible :=           b;
      aIm_hero[i].hand_up.Visible :=        b;
      aIm_hero[i].hand_down.Visible :=      b;
      aIm_hero[i].leg_left_up.Visible :=    b;
      aIm_hero[i].leg_left_down.Visible :=  b;
      aIm_hero[i].leg_right_up.Visible :=   b;
      aIm_hero[i].leg_right_down.Visible := b;
      aIm_hero[i].base.Visible :=           b;
    end;
end;





procedure TForm1.Vid_Show_All_Items(b: boolean);
{ displays/hides all items                                                     }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 7 do
      aIm_item[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Flames(b: boolean);
{ displays/hides all flames                                                    }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    for j := 1 to 7 do
      aIm_flame[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Character_2(b: boolean);
{ displays/hides all the sprites of "Character No. 2"                          }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_character_2[i].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := i <= p;                                        // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ displays score in image format                                               }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t > 99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >  9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                     aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].picture;                                            // units
    end
  else                                                                            // hide
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Hero(var p, f: integer; b: boolean);
{ displays/hides the hero in "p" position; when visible, displays him with     }
{ left/right legs up/down according to the "f" parameter                       }
begin
  aIm_hero[p].body.Visible :=           b;
  aIm_hero[p].hand_down.Visible :=      b;
  aIm_hero[p].leg_left_down.Visible :=  b and Odd(f);
  aIm_hero[p].leg_left_up.Visible :=    b and (not Odd(f));
  aIm_hero[p].leg_right_down.Visible := b and (not Odd(f));
  aIm_hero[p].leg_right_up.Visible :=   b and Odd(f);
  aIm_hero[p].base.Visible :=           b;
end;





procedure TForm1.Vid_Show_Items;
{ displays normal items sprites, depending on values of the aItem array, and   }
{ hides sprites in every other position                                        }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 7 do
      aIm_item[i, j].Visible := aItem[i] = j;                                   // displays 1 item per column
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_background_bottomMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_background_bottomMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_openMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Open button: PRESS                                                           }
begin
  if Button = mbLeft then
     Inp_open_down;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then                                                   // Open or ACL
    if (iMode = 0) then
      Inp_open_down                                                               // Open
    else
      Inp_acl_down else                                                           // ACL
  if Key = aKeys[5].word then Inp_left_down   else                              // Left
  if Key = aKeys[6].word then Inp_right_down;                                   // Right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if (Key = aKeys[4].word) and (iMode > 0) then Inp_acl_up else                 // ACL

  if Key = aKeys[5].word then Inp_left_up   else                                // Left
  if Key = aKeys[6].word then Inp_right_up;                                     // Right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_open_down;
{ Open: PRESS                                                                  }
begin
  btn_open.Visible := false;
  im_background_front.Visible := false;
  im_background_back.Visible := true;
  im_background_screen_top.Visible := true;

  iMode := 2; Gam_Set_Mode;                                                     // closed -> ACL/2
end;





procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ Game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 5 then                                                             // hiscore A ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ Game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ Game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 7 then                                                             // hiscore B ->
    begin
      iMode := 8; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 3 then                                                             // alarm ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  if iMode <> 0 then                                                            // any mode except closed ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //                      -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 1 then                                                             // ACL/1 ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_left_down;
{ Left button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if (bControls and (not bKeyPressed)) and (iPosHeroX > 1) then
    begin
      bKeyPressed := true;

      aT_hand[iPosHeroX].Enabled := false;

      Vid_Show_Hero(iPosHeroX, iLoopGame, false);
      aIm_hero[iPosHeroX].hand_up.Visible := false;

      Dec(iPosHeroX);

      Vid_Show_Hero(iPosHeroX, iLoopGame, true);

      Snd_Play(pfs_step);
    end;

  Gam_Check_Collision_Hand_Item;
end;





procedure TForm1.Inp_left_up;
{ Left button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ Right button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if (bControls and (not bKeyPressed)) and (iPosHeroX < 4) then
    begin
      bKeyPressed := true;

      aT_hand[iPosHeroX].Enabled := false;

      Vid_Show_Hero(iPosHeroX, iLoopGame, false);
      aIm_hero[iPosHeroX].hand_up.Visible := false;

      Inc(iPosHeroX);

      Vid_Show_Hero(iPosHeroX, iLoopGame, true);

      Snd_Play(pfs_step);
    end;

  Gam_Check_Collision_Hand_Item;
end;





procedure TForm1.Inp_right_up;
{ Right button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;
  bKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
  var
    i: integer;
begin
  Timer_Chance_Time.Enabled := false;
  Timer_Clock.Enabled       := false;
  Timer_Game.Enabled        := false;
  Timer_Game_Over.Enabled   := false;
  Timer_Game_Start.Enabled  := false;
  Timer_Miss_1.Enabled      := false;
  Timer_Miss_2.Enabled      := false;
  Timer_Miss_3.Enabled      := false;
  Timer_Score_Blink.Enabled := false;

  for i := 1 to 4 do
    aT_hand[i].Enabled      := false;
end;





procedure TForm1.Gam_Check_Chance_Time;
{ checks whether chance time score has been achieved                           }
begin
  if iScore >= iScoreNextChance then
    begin
      Timer_Game.Enabled := false;
      Snd_Silence;

      bControls := false;

      Inc(iScoreNextChance, 1000);

      iLoopChanceTime := 1;
      Timer_Chance_Time.Interval := 1;
      Timer_Chance_Time.Enabled := true;
    end;
end;





procedure TForm1.Gam_Check_Collision_Hand_Item;
{ checks collision between hand and normal item -> scores points and then      }
{ plays "beep 2"                                                               }
  var
    bGot: boolean;
begin
  bGot := false;

  case iPosHeroX of
    1: if aItem[1] = 5 then begin aItemTarget[1] := Random(2) + 1; bGot := true; end;
 2, 3: if aItem[2] = 5 then begin aItemTarget[2] := Random(2) + 1; bGot := true; end;
    4: if aItem[3] = 5 then begin aItemTarget[3] := Random(2) + 1; bGot := true; end;
  end; // case..of

  if bGot then
    begin
      aT_hand[iPosHeroX].Interval := 80;
      aT_hand[iPosHeroX].Enabled := true;

      Snd_Play(pfs_beep2);

      Inc(iScore, iScoreMultiplier);
      Vid_Show_Panel_Score(iScore, true);
      Gam_Set_Difficulty;

      Gam_Check_Chance_Time;
    end;
end;





procedure TForm1.Gam_Check_Collision_Item_Hand;
{ checks collision between normal item and hand -> scores points and then      }
{ plays "beep 3"                                                               }
  var
    bGot: boolean;
begin
  bGot := false;

  case iPosHeroX of
    1: if aItem[1] = 5 then begin aItemTarget[1] := Random(2) + 1; bGot := true; end;
 2, 3: if aItem[2] = 5 then begin aItemTarget[2] := Random(2) + 1; bGot := true; end;
    4: if aItem[3] = 5 then begin aItemTarget[3] := Random(2) + 1; bGot := true; end;
  end; // case..of

  if bGot then
    begin
      aT_hand[iPosHeroX].Interval := 80;
      aT_hand[iPosHeroX].Enabled := true;

      Snd_Play(pfs_beep3);

      Inc(iScore, iScoreMultiplier);
      Vid_Show_Panel_Score(iScore, true);
      Gam_Set_Difficulty;

      Gam_Check_Chance_Time;
    end;
end;





procedure TForm1.Gam_Set_Difficulty;
{ calculates difficulty increase, depending on current score                   }
  var
    i, j,
    iHundreds,
    iTens:     integer;
begin
  i := iScore;
  iHundreds := 0;
  iTens := 0;

  // calculates how many hundreds and tens in iScore
  while i > 99 do
    begin
      Dec(i, 100);
      Inc(iHundreds);
    end;

  while i > 9 do
    begin
      Dec(i, 10);
      Inc(iTens);
    end;

  // sets random number for flames generator (PRESUMED!)
  j := 12 - iHundreds - iTens;

  if j < 3 then
    j := 3;

  iMaxRandomFlames := j;

  // sets game speed (PRESUMED!)
  j := TIMER_GAME_NORMAL - (iHundreds + iTens) * 24;

  if j < 320 then
    j := 320;

  Timer_Game.Interval := j;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/ACL mode, depending on the value of the iMode variable  }
begin
  case iMode of
    0: begin                                                                    // handheld is closed
       end;
    1: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         Snd_Silence;

         bControls := false;
       end;
    2: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    3: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_game_background.Visible := true;
         aIm_character_2[1].Visible := true;
         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;
         aIm_panel_digit_pos[1].picture := aIm_panel_number[1].picture;
         aIm_panel_digit_pos[2].picture := aIm_panel_number[2].picture;
         aIm_panel_digit_pos[3].picture := aIm_panel_number[0].picture;
         aIm_panel_digit_pos[4].picture := aIm_panel_number[0].picture;
       end;
    4: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_game_background.Visible := true;
         im_panel_time_colon.Visible := true;
         aIm_character_2[1].Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;
       end;
 5, 7: begin                                                                    // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         case iMode of
           5: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of
       end;
 6, 8: begin                                                                    // GAME A/B (Game A/B button release)
         Gam_Stop_All_Timers;

         im_game_background.Visible := true;
         aIm_character_2[1].Visible := true;

         iMiss := 0;
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         iScoreMultiplier := 1;
         iScoreNextChance := 300;

         Gam_Set_Difficulty;

         iLoopGameStart := 1;
         Timer_Game_Start.Interval := 1;
         Timer_Game_Start.Enabled := true;
       end;
    9: begin                                                                    // GAME OVER
         // does nothing
       end;
    end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Chance_TimeTimer(Sender: TObject);
{ plays "chance" tune then game action reprises                                }
begin
  case iLoopChanceTime of
    1: begin
         Snd_Play(pfs_chance);

         Timer_Chance_Time.Interval := 3800;
       end;
    2: begin
         if iMiss = 0 then
           begin
             iScoreMultiplier := 2;

             iLoopScoreBlink := 1;
             Timer_Score_Blink.Interval := 500;
             Timer_Score_Blink.Enabled := true;
           end;

         iMiss := 0; Vid_Show_Panel_Miss(iMiss);

         bControls := true;
         Timer_Game.Enabled := true;

         Timer_Chance_Time.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopChanceTime);
end;





procedure TForm1.Timer_ClockTimer(Sender:TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
var
  i, j: integer;
  iRndCol, iRndComb: integer;
  bMiss, bPlaySound: boolean;
begin
  // hero's animation frame update (legs)
  Vid_Show_Hero(iPosHeroX, iLoopGame, true);

  // items move, collisions, points, misses, etc.
  if Odd(iLoopGame) then                                                        // iLoopGame is odd
    begin
      // normal items move
      for i := 1 to 3 do
        if aItem[i] = aItemTarget[i] then
          aItemTarget[i] := 6
        else
          begin
            aIm_item[i, aItem[i]].Visible := false;

            if aItem[i] > aItemTarget[i] then
              Dec(aItem[i])
            else
              Inc(aItem[i]);

            aIm_item[i, aItem[i]].Visible := true;
          end;

      Snd_Play(pfs_beep1);

      // checks for collision hand-normal items (miss)
      bMiss := false;

      for i := 1 to 3 do
        if aItem[i] = 6 then
          bMiss := true;

      if bMiss then
        begin
          Gam_Stop_All_Timers;
          bControls := false;

          Vid_Show_Panel_Score(iScore, true);
          Vid_Show_All_Flames(false);

          for i := 1 to 3 do
            for j := 1 to 5 do
              aIm_item[i, j].Visible := false;

          Snd_Silence;
          Snd_Play(pfs_miss1);

          iLoopMiss3FlameX := 0;                                                // prevents flaming items from blinking
          iLoopMiss1 := 1;
          Timer_Miss_1.Interval := 1;
          Timer_Miss_1.Enabled := true;
        end;

      // checks for collision hand-normal items (point)
      if not bMiss then
        Gam_Check_Collision_Item_Hand;
    end // Odd(iLoopGame)
  else                                                                          // iLoopGame is even
    // flaming items move
    begin
      // checks for collision hand-flaming items (miss)
      iLoopMiss3FlameX := 0;

      if aIm_flame[1, 5].Visible and (iPosHeroX < 3) then
        iLoopMiss3FlameX := 1;

      if aIm_flame[2, 5].Visible and (iPosHeroX > 2) then
        iLoopMiss3FlameX := 2;

      if iLoopMiss3FlameX <> 0 then
        begin
          // a miss has occured!
          Gam_Stop_All_Timers;
          bControls := false;

          Vid_Show_Panel_Score(iScore, true);
          Vid_Show_All_Items(false);
          Vid_Show_All_Flames(false);
          aIm_flame[iLoopMiss3FlameX, 5].Visible := true;

          aIm_hero[iPosHeroX].hand_up.Visible := false;
          aIm_hero[iPosHeroX].hand_down.Visible := true;

          Snd_Silence;
          Snd_Play(pfs_miss3);

          iLoopMiss3 := 1;
          Timer_Miss_3.Interval := 1;
          Timer_Miss_3.Enabled := true;
        end
      else
        // no miss has occured -> flaming items move
        begin
          bPlaySound := false;

          for i := 1 to 2 do
            begin
              if aIm_flame[i, 7].Visible then
                aIm_flame[i, 7].Visible := false;

              for j := 6 downto 1 do
                if aIm_flame[i, j].Visible then
                  begin
                    aIm_flame[i, j + 1].Visible := true;
                    aIm_flame[i, j].Visible := false;
                    bPlaySound := true;
                  end;

                if bPlaySound then
                  Snd_Play(pfs_beep4);
            end;

          case iMode of
            6: if (aIm_flame[1, 1].Visible = aIm_flame[1, 2].Visible) and
                  (aIm_flame[2, 1].Visible = aIm_flame[2, 2].Visible) then
                 begin
                   iRndCol := Random(2) + 1;
                   iRndComb:= Random(iMaxRandomFlames);

                   if iRndComb < 2 then
                     begin
                       case iRndCol of
                         1: if aIm_flame[1, 3].Visible then iRndCol := 2;
                         2: if aIm_flame[2, 3].Visible then iRndCol := 1;
                       end; // case..of

                       aIm_flame[iRndCol, 1].Visible := true;
                     end;
                 end;
            8: begin
                 iRndCol := Random(2) + 1;
                 iRndComb:= Random(iMaxRandomFlames);

                 if iRndComb < 3 then
                   if (aIm_flame[iRndCol, 1].Visible = aIm_flame[iRndCol, 2].Visible) and
                      (aIm_flame[iRndCol, 2].Visible = aIm_flame[iRndCol, 3].Visible) then
                       aIm_flame[iRndCol, 1].Visible := true;
               end;
          end; // case..of
        end;
    end; // not Odd(iLoopGame)

  // calculates speed/difficulty for next Timer_Game tick
  Gam_Set_Difficulty;

  Inc(iLoopGame);

  if iLoopGame = 3 then
    iLoopGame := 1;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ Donkey Kong and Mario jump for 5 minutes, then mode is set back to "time"    }
begin
  if iLoopGameOver = 1 then
    begin
      while iScore >= 1000 do
       Dec(iScore, 1000);

      if (iMode = 6) and (iScore > iHiScoreA) then
       begin
         iHiScoreA := iScore; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
       end;

      if (iMode = 8) and (iScore > iHiScoreB) then
       begin
         iHiScoreB := iScore; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
       end;

      iMode := 9; Gam_Set_Mode;
    end;

  aIm_character_2[1].Visible := Odd(iLoopGameOver);
  aIm_character_2[2].Visible := not Odd(iLoopGameOver);

  aIm_hero[iPosHeroX].leg_left_down.Visible := Odd(iLoopGameOver);
  aIm_hero[iPosHeroX].leg_left_up.Visible := not Odd(iLoopGameOver);
  aIm_hero[iPosHeroX].leg_right_down.Visible := Odd(iLoopGameOver);
  aIm_hero[iPosHeroX].leg_right_up.Visible := not Odd(iLoopGameOver);

  Inc(iLoopGameOver);

  // sets the game to "Time" mode
  if iLoopGameOver = 375 then
    begin
      iMode := 4; Gam_Set_Mode;

      Timer_Game_Over.Enabled := false;
    end;

  Timer_Game_Over.Interval := 800;
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ game start animation                                                         }
begin
  case iLoopGameStart of
    1: begin
         Snd_Play(pfs_start);

         Timer_Game_Start.Interval := 300;
       end;
    2: begin
        aIm_hero[1].body.Visible := true;  aIm_hero[1].hand_down.Visible := true;  aIm_hero[1].leg_left_down.Visible := true;  aIm_hero[1].leg_right_up.Visible := true;    aIm_hero[1].base.Visible := true;

        Timer_Game_Start.Interval := 375;
       end;
    3: begin
        Vid_Show_All_Hero(false);
        aIm_hero[2].body.Visible := true;  aIm_hero[2].hand_down.Visible := true;  aIm_hero[2].leg_left_down.Visible := true;  aIm_hero[2].leg_right_up.Visible := true;    aIm_hero[2].base.Visible := true;
       end;
    4: begin
        Vid_Show_All_Hero(false);
        aIm_hero[3].body.Visible := true;  aIm_hero[3].hand_down.Visible := true;  aIm_hero[3].leg_left_up.Visible := true;    aIm_hero[3].leg_right_down.Visible := true;  aIm_hero[3].base.Visible := true;
       end;
    5: begin
        Vid_Show_All_Hero(false);
        aIm_hero[4].body.Visible := true;  aIm_hero[4].hand_up.Visible := true;    aIm_hero[4].leg_left_down.Visible := true;  aim_hero[4].leg_right_down.Visible := true;  aIm_hero[4].base.Visible := true;
       end;
    6: begin
        Vid_Show_All_Hero(false);
        aIm_hero[3].body.Visible := true;  aIm_hero[3].hand_down.Visible := true;  aIm_hero[3].leg_left_up.Visible := true;    aIm_hero[3].leg_right_down.Visible := true;  aIm_hero[3].base.Visible := true;
       end;
    7: begin
        Vid_Show_All_Hero(false);
        aIm_hero[2].body.Visible := true;  aIm_hero[2].hand_up.Visible   := true;  aIm_hero[2].leg_left_down.Visible := true;  aIm_hero[2].leg_right_down.Visible := true;  aIm_hero[2].base.Visible := true;
       end;
    8: begin
                                           aIm_hero[2].hand_up.Visible := false;                                               aIm_hero[2].leg_right_down.Visible := false;
                                           aIm_hero[2].hand_down.Visible := true;                                              aIm_hero[2].leg_right_up.Visible := true;

        iLoopGame := 1;
        Timer_Game.Interval := TIMER_GAME_NORMAL;
        Timer_Game.Enabled := true;
       end;
    9: begin
        aItem[1] := 3;
        aItem[2] := 3;
        aItem[3] := 1;
        Vid_Show_Items;

        aItemTarget[1] := 1;
        aItemTarget[2] := 6;
        aItemTarget[3] := 6;

        aIm_flame[1, 1].Visible := true;

        iPosHeroX := 2;

        bControls := true;
        bKeyPressed := false;

        Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_Hero_Hand_1Timer(Sender: TObject);
{ collision check for the hero's hand in column 1                              }
begin
  if not aIm_hero[1].hand_up.Visible then
    begin
      aIm_hero[1].hand_down.Visible := false;
      aIm_hero[1].hand_up.Visible := true;

      aIm_item[1, 5].Visible := false;
      aIm_item[1, 4].Visible := true;
      aItem[1] := 4;
    end
  else
    begin
      aIm_hero[1].hand_up.Visible := false;
      aIm_hero[iPosHeroX].hand_down.Visible := true;

      aT_hand[1].Enabled := false;
    end;
end;





procedure TForm1.Timer_Hero_Hand_2Timer(Sender: TObject);
{ collision check for the hero's hand in column 2                              }
begin
  if not aIm_hero[2].hand_up.Visible then
    begin
      aIm_hero[2].hand_down.Visible := false;
      aIm_hero[2].hand_up.Visible := true;

      aIm_item[2, 5].Visible := false;
      aIm_item[2, 4].Visible := true;
      aItem[2] := 4;
    end
  else
    begin
      aIm_hero[2].hand_up.Visible := false;
      aIm_hero[iPosHeroX].hand_down.Visible := true;

      aT_hand[2].Enabled := false;
    end;
end;





procedure TForm1.Timer_Hero_Hand_3Timer(Sender: TObject);
{ collision check for the hero's hand in column 3                              }
begin
  if not aIm_hero[3].hand_up.Visible then
    begin
      aIm_hero[3].hand_down.Visible := false;
      aIm_hero[3].hand_up.Visible := true;

      aIm_item[2, 5].Visible := false;
      aIm_item[2, 4].Visible := true;
      aItem[2] := 4;
    end
  else
    begin
      aIm_hero[3].hand_up.Visible := false;
      aIm_hero[iPosHeroX].hand_down.Visible := true;

      aT_hand[3].Enabled := false;
    end;
end;





procedure TForm1.Timer_Hero_Hand_4Timer(Sender: TObject);
{ collision check for the hero's hand in column 4                              }
begin
  if not aIm_hero[4].hand_up.Visible then
    begin
      aIm_hero[4].hand_down.Visible := false;
      aIm_hero[4].hand_up.Visible := true;

      aIm_item[3, 5].Visible := false;
      aIm_item[3, 4].Visible := true;
      aItem[3] := 4;
    end
  else
    begin
      aIm_hero[4].hand_up.Visible := false;
      aIm_hero[iPosHeroX].hand_down.Visible := true;

      aT_hand[4].Enabled := false;
    end;
end;





procedure TForm1.Timer_Miss_1Timer(Sender: TObject);
{ phase 1 of miss animation (normal item falls down)                          }
  var
    i: integer;
begin
  case iLoopMiss1 of
    1: Timer_Miss_1.Interval := 280;
    2: begin
         for i := 1 to 3 do
           begin
             aIm_item[i, 7].Visible := aIm_item[i, 6].Visible;
             aIm_item[i, 6].Visible := false;
           end;

         iLoopMiss2 := 1;
         Timer_Miss_2.Interval := 1600;
         Timer_Miss_2.Enabled := true;

         Timer_Miss_1.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss1);
end;





procedure TForm1.Timer_Miss_2Timer(Sender: TObject);
{ phase 2 of miss animation (the hero moves to position X = 4)                 }
begin
  if iPosHeroX < 4 then
    // the hero moves right
    begin
      Snd_Silence;
      Snd_Play(pfs_step);

      Vid_Show_All_Hero(false);
      Inc(iPosHeroX);
      aIm_hero[iPosHeroX].body.Visible := true;
      aIm_hero[iPosHeroX].hand_down.Visible := true;
      aIm_hero[iPosHeroX].leg_left_down.Visible := true;
      aIm_hero[iPosHeroX].leg_right_down.Visible := true;
      aIm_hero[iPosHeroX].base.Visible := true;
    end
  else
    // the hero is in X=4, the intermission continues
    begin
      Snd_Play(pfs_miss2);

      iLoopMiss3 := 1;
      Timer_Miss_3.Interval := 1;
      Timer_Miss_3.Enabled := true;

      Timer_Miss_2.Enabled := false;
    end;

  Timer_Miss_2.Interval := 488;
end;





procedure TForm1.Timer_Miss_3Timer(Sender: TObject);
{ phase 3 of miss animation (Donkey Kong and Mario jump)                       }
begin
  case iLoopMiss3 of
    1: Timer_Miss_3.Interval := 200;
    2: begin
         aIm_hero[iPosHeroX].leg_left_down.Visible := false; aIm_hero[iPosHeroX].leg_right_down.Visible := false; aIm_character_2[1].Visible := false;
         aIm_hero[iPosHeroX].leg_left_up.Visible := true;    aIm_hero[iPosHeroX].leg_right_up.Visible := true;    aIm_character_2[2].Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := false;
       end;
    3: begin
         aIm_hero[iPosHeroX].leg_left_up.Visible := false;  aIm_hero[iPosHeroX].leg_right_up.Visible := false;
         aIm_hero[iPosHeroX].leg_left_down.Visible := true; aIm_hero[iPosHeroX].leg_right_down.Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := true;
       end;
    4: begin
         aIm_hero[iPosHeroX].leg_right_down.Visible := false; aIm_character_2[2].Visible := false;
         aIm_hero[iPosHeroX].leg_right_up.Visible := true;    aIm_character_2[1].Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := false;
       end;
    5: begin
         aIm_hero[iPosHeroX].leg_right_up.Visible := false;
         aIm_hero[iPosHeroX].leg_right_down.Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := true;
       end;
    6: begin
         aIm_hero[iPosHeroX].leg_left_down.Visible := false; aIm_character_2[1].Visible := false;
         aIm_hero[iPosHeroX].leg_left_up.Visible := true; aIm_character_2[2].Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := false;
       end;
    7: begin
         aIm_hero[iPosHeroX].leg_left_up.Visible := false;
         aIm_hero[iPosHeroX].leg_left_down.Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := true;
       end;
    8: begin
         aIm_hero[iPosHeroX].leg_left_down.Visible := false; aIm_hero[iPosHeroX].leg_right_down.Visible := false; aIm_character_2[2].Visible := false;
         aIm_hero[iPosHeroX].leg_left_up.Visible := true;    aIm_hero[iPosHeroX].leg_right_up.Visible := true;    aIm_character_2[1].Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := false;
       end;
    9: begin
         aIm_hero[iPosHeroX].leg_left_up.Visible := false;   aIm_hero[iPosHeroX].leg_right_up.Visible := false;
         aIm_hero[iPosHeroX].leg_left_down.Visible := true;  aIm_hero[iPosHeroX].leg_right_down.Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := true;
       end;
   10: begin
         aIm_hero[iPosHeroX].leg_right_down.Visible := false; aIm_character_2[1].Visible := false;
         aIm_hero[iPosHeroX].leg_right_up.Visible := true; aIm_character_2[2].Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := false;
       end;
   11: begin
         aIm_hero[iPosHeroX].leg_right_down.Visible := true;
         aIm_hero[iPosHeroX].leg_right_up.Visible := false;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := true;
       end;
   12: begin
         aIm_character_2[2].Visible := false;
         aIm_character_2[1].Visible := true;

         if iLoopMiss3FlameX <> 0 then aIm_flame[iLoopMiss3FlameX, 5].Visible := false;

         Timer_Miss_3.Interval := 600;
       end;
   13: begin
         Inc(iMiss);
         Vid_Show_Panel_Miss(iMiss);

         if iMiss = 3 then                                                      // game is over
           begin
             Snd_Play(pfs_gameover);

             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 3800;
             Timer_Game_Over.Enabled := true;
           end
         else                                                                   // game reprises
           begin
             aIm_hero[iPosHeroX].leg_right_down.Visible := false;
             aIm_hero[iPosHeroX].leg_right_up.Visible := true;

             aItem[1] := 3;
             aItem[2] := 3;
             aItem[3] := 1;
             Vid_Show_Items;

             aItemTarget[1] := 1;
             aItemTarget[2] := 6;
             aItemTarget[3] := 6;

             aIm_flame[1, 1].Visible := true;

             iScoreMultiplier := 1;

             bControls := true;

             iLoopGame := 1;
             Timer_Game.Enabled := true;
           end;

         Timer_Miss_3.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss3);
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score blinks during chance time                                              }
begin
  Vid_Show_Panel_Score(iScore, not Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink >= 3 then
    iLoopScoreBlink := 1;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
